| package |
package := Package name: 'SSW Modal'.
package paxVersion: 0;
	basicComment: ' Solutions Software 2004

You may freely use or modify this code in your own commercial or non-commercial applications. 

Use at your own risk: no warranty is provided as to the suitability of the software for any particular task, and no responsibility is assumed by the authors for any loss or damage that may result from its use.

By use of the software, you are deemed to have agreed to these terms & conditions.

******

Examples - copy to a workspace:

"Modally open a Calculator shell"
Calculator new showModal

"Modally open a Calculator shell, locking the whole image"
Calculator new showTaskModal

"Modally open a Scribble presenter - answer true for OK, false for Cancel"
Scribble new showModal

"Modally open a Scribble presenter, answering the number of strokes if OK''d"
Scribble new 
	showModalTo: View active presenter 
	onOK: [ :scribble | scribble model size] 
	onCancel: [-1]


'.


package classNames
	add: #SurrogateDialog;
	yourself.

package methodNames
	add: #Presenter -> #caption;
	add: #Presenter -> #caption:;
	add: #Presenter -> #show:modalTo:onOK:onCancel:;
	add: #Presenter -> #showModal;
	add: #Presenter -> #showModalTo:;
	add: #Presenter -> #showModalTo:onOK:onCancel:;
	add: #Presenter -> #showTaskModal:onOK:onCancel:;
	add: #Presenter -> #showTaskModalOnOK:onCancel:;
	add: #Shell -> #disableWithAll:;
	add: #Shell -> #onModalShellActivatedFor:;
	add: #Shell -> #onModalShellCloseRequested:;
	add: #Shell -> #reEnable:windows:;
	add: #Shell -> #show:modalTo:;
	add: #Shell -> #show:modalTo:disabling:;
	add: #Shell -> #showMaximizedTaskModal:;
	add: #Shell -> #showModal;
	add: #Shell -> #showModalTo:;
	add: #Shell -> #showTaskModal;
	add: #Shell -> #showTaskModal:;
	yourself.

package binaryGlobalNames: (Set new
	yourself).

package globalAliases: (Set new
	yourself).

package allResourceNames: (Set new
	add: #SurrogateDialog -> 'Default view';
	yourself).

package setPrerequisites: (IdentitySet new
	add: '..\Object Arts\Dolphin\Base\Dolphin';
	add: '..\Object Arts\Dolphin\MVP\Base\Dolphin MVP Base';
	yourself).

package!

"Class Definitions"!

Shell subclass: #SurrogateDialog
	instanceVariableNames: 'containerPresenter containedPresenter viewName okBlock cancelBlock result isClosing'
	classVariableNames: ''
	poolDictionaries: ''
	classInstanceVariableNames: ''!

"Global Aliases"!


"Loose Methods"!

!Presenter methodsFor!

caption

	"For when opened modally"

	^self propertyAt: #modalCaption ifAbsent: ['']!

caption: aString

	"For when opened modally"

	self propertyAt: #modalCaption put: aString!

show: aViewName modalTo: aShell onOK: okBlock onCancel: cancelBlock

	^SurrogateDialog new
		containedPresenter: self;
		viewName: aViewName;
		okBlock: okBlock;
		cancelBlock: cancelBlock;
		caption: self caption;
		showModalTo: aShell;
		result!

showModal

	"Allow a regular Shell to be opened Modal to another
	 - useful to enforce transaction boundaries"

	^self showModalTo: View active presenter
!

showModalTo: aShell

	^self showModalTo: aShell onOK: [true] onCancel: [false]!

showModalTo: aShell onOK: okBlock onCancel: cancelBlock

	^self show: self class defaultView modalTo: aShell onOK: okBlock onCancel: cancelBlock
!

showTaskModal: aViewName onOK: okBlock onCancel: cancelBlock

	^SurrogateDialog new
		containedPresenter: self;
		viewName: aViewName;
		okBlock: okBlock;
		cancelBlock: cancelBlock;
		showTaskModal;
		result!

showTaskModalOnOK: okBlock onCancel: cancelBlock

	^self showTaskModal: self class defaultView onOK: okBlock onCancel: cancelBlock
! !
!Presenter categoriesFor: #caption!accessing!public! !
!Presenter categoriesFor: #caption:!accessing!public! !
!Presenter categoriesFor: #show:modalTo:onOK:onCancel:!operations!public! !
!Presenter categoriesFor: #showModal!operations!public! !
!Presenter categoriesFor: #showModalTo:!operations!public! !
!Presenter categoriesFor: #showModalTo:onOK:onCancel:!operations!public! !
!Presenter categoriesFor: #showTaskModal:onOK:onCancel:!operations!public! !
!Presenter categoriesFor: #showTaskModalOnOK:onCancel:!operations!public! !

!Shell methodsFor!

disableWithAll: aCollection

	self view disable.
	aCollection do: [ :each | each view disable]!

onModalShellActivatedFor: aView

	aView isOpen ifTrue: 
		[self view disable]!

onModalShellCloseRequested: aValueHolder

	"Ensure this shell will have the focus when the modal child closes"

	self view beForeground!

reEnable: sem windows: aCollection

	"Re-enable the receiver and its fellows following closure of a spawned modal Shell"

	[self view enable. 
	aCollection do: [ :each | each view enable].
	self view lastFocus ifNotNil: [ :focusView | focusView setFocus].
	sem signal] postToInputQueue!

show: aViewName modalTo: aShell

	"Allow a regular Shell to be opened Modal to another
	 - useful to enforce transaction boundaries"

	self show: aViewName modalTo: aShell disabling: #()!

show: aViewName modalTo: aShell disabling: windows

	"Allow a regular Shell to be opened Modal to another
	 - useful to enforce transaction boundaries"

	| focusView toolboxView sem wasMain proc |

	focusView := aShell view subViews detect: [ :e | e hasFocus] ifNone: [nil].
	toolboxView := self class loadViewResource: aViewName inContext: View desktop.
	toolboxView bePopupFor: aShell view.
	self view: toolboxView.
	self caption notNil ifTrue: [toolboxView caption: self caption].

	sem := Semaphore new.

	[self 
		when: #closeRequested: send: #onModalShellCloseRequested: to: aShell;
		when: #viewClosed send: #reEnable:windows: to: aShell withArguments: (Array with: sem with: windows).
	self view 
		setWindowStyle: ##(DialogView basicNew defaultWindowStyle); 
		extent: (self view extent - (1@1));
		when: #viewActivated send: #disableWithAll: to: aShell with: windows.
	aShell view when: #viewActivated send: #onModalShellActivatedFor: to: aShell with: self view.
	self show] 
		on: Error do: [ :exc | aShell view enable. exc signal].

	"Based on DialogView>>showModal"
	wasMain := Processor forkMainIfMain.
	wasMain ifTrue: 
		["Boost the initiating UI processes priority so that the operation requested by the user
		completes more quickly when the dialog is closed."
		proc := Processor activeProcess.
		proc priority: proc priority + 1].

	sem wait!

showMaximizedTaskModal: aViewName

	"Allow a regular Shell to be opened Modal to all other windows
	 - useful to enforce transaction boundaries"

	| windows sem wasMain proc |

	windows := SessionManager current inputState topLevelWindows.
	windows := windows select: [ :each | each view isEnabled].
	self view: (self class loadViewResource: aViewName inContext: View desktop).
	sem := Semaphore new.

	[self 
		when: #viewClosed send: #reEnable:windows: to: self withArguments: (Array with: sem with: windows).
	windows do: [ :each | each view disable].
	self view 
		setWindowStyle: ##(DialogView basicNew defaultWindowStyle); 
		extent: (self view extent - (1@1));
		showMaximized] 
			on: Error do: [ :exc | windows do: [ :each | each view enable]. exc signal].

	"Based on DialogView>>showModal"
	wasMain := Processor forkMainIfMain.
	wasMain ifTrue: 
		["Boost the initiating UI processes priority so that the operation requested by the user
		completes more quickly when the dialog is closed."
		proc := Processor activeProcess.
		proc priority: proc priority + 1].

	sem wait!

showModal

	"Allow a regular Shell to be opened Modal to another
	 - useful to enforce transaction boundaries"

	self showModalTo: View active presenter!

showModalTo: aShell

	"Allow a regular Shell to be opened Modal to another
	 - useful to enforce transaction boundaries"

	self show: self class defaultView modalTo: aShell!

showTaskModal

	self showTaskModal: self class defaultView!

showTaskModal: aViewName

	"Allow a regular Shell to be opened Modal to all other windows
	 - useful to enforce transaction boundaries"

	| activeWindow otherWindows |

	activeWindow := View active presenter.
	otherWindows := SessionManager current inputState topLevelWindows.
	otherWindows := otherWindows select: [ :each | each ~~ activeWindow and: [each view isEnabled]].

	^self show: aViewName modalTo: activeWindow disabling: otherWindows! !
!Shell categoriesFor: #disableWithAll:!accessing!public! !
!Shell categoriesFor: #onModalShellActivatedFor:!accessing!public! !
!Shell categoriesFor: #onModalShellCloseRequested:!accessing!public! !
!Shell categoriesFor: #reEnable:windows:!accessing!public! !
!Shell categoriesFor: #show:modalTo:!accessing!public! !
!Shell categoriesFor: #show:modalTo:disabling:!accessing!public! !
!Shell categoriesFor: #showMaximizedTaskModal:!accessing!public! !
!Shell categoriesFor: #showModal!accessing!public! !
!Shell categoriesFor: #showModalTo:!accessing!public! !
!Shell categoriesFor: #showTaskModal!accessing!public! !
!Shell categoriesFor: #showTaskModal:!accessing!public! !

"End of package definition"!

"Source Globals"!

"Classes"!

SurrogateDialog guid: (GUID fromString: '{ED512DED-DA8D-4352-8D66-09DB85D82A5E}')!
SurrogateDialog comment: ''!
!SurrogateDialog categoriesForClass!MVP-Presenters! !
!SurrogateDialog methodsFor!

additionalAccelerators

	^#(#(#cancel 'ESC'))!

cancel

	self cancelBlock argumentCount = 0
		ifTrue: [self result: self cancelBlock value]
		ifFalse: [self result: (self cancelBlock value: self containedPresenter)].

	self isClosing: true.

	^self exit!

cancelBlock

	"cancelBlock is evaluated with on Cancel, 
	the result of the evaluation being stored in the result i.v. of the receiver.

	cancelBlock may be zero or 1-arg, in the latter case the argument is the contained presenter"

	^cancelBlock!

cancelBlock: anObject

	"cancelBlock is evaluated with on Cancel, 
	the result of the evaluation being stored in the result i.v. of the receiver.

	cancelBlock may be zero or 1-arg, in the latter case the argument is the contained presenter"

	cancelBlock := anObject!

containedPresenter
	^containedPresenter!

containedPresenter: anObject
	containedPresenter := anObject!

createComponents

	super createComponents.
	containerPresenter := self add: Presenter new name: 'container'!

createSchematicWiring

	super createSchematicWiring.

	self when: #closeRequested: send: #onCloseRequested: to: self!

initialize

	super initialize.
	self isClosing: false!

isClosing
	^isClosing!

isClosing: anObject
	isClosing := anObject!

ok

	self okBlock argumentCount = 0
		ifTrue: [self result: self okBlock value]
		ifFalse: [self result: (self okBlock value: self containedPresenter)].

	self isClosing: true.
	^self exit!

okBlock

	"okBlock is evaluated with on OK, 
	the result of the evaluation being stored in the result i.v. of the receiver.

	okBlock may be zero or 1-arg, in the latter case the argument is the contained presenter"

	^okBlock!

okBlock: anObject

	"okBlock is evaluated with on OK, 
	the result of the evaluation being stored in the result i.v. of the receiver.

	okBlock may be zero or 1-arg, in the latter case the argument is the contained presenter"

	okBlock := anObject!

onCloseRequested: aValueHolder

	"Turn an external close into a cancel"

	aValueHolder value: self isClosing.

	aValueHolder value ifFalse: [[self cancel] postToInputQueue]!

onViewOpened

	| contained |

	super onViewOpened.
	[self view updateCombinedAcceleratorTable] postToInputQueue.

	(containerPresenter add: self containedPresenter name: 'contained')
		createView: self viewName;
		onViewOpened.	

	self view 
		caption: self containedPresenter view text;
		extent: (self containedPresenter view extent + (20@78));
		position: (self view centerExtent: self view extent within: View desktop)
	!

queryCommand: aCommand

	| cmd |

	cmd := aCommand commandSymbol.

	(cmd = #ok and: [self containedPresenter queryCommand: aCommand]) ifTrue: [^true].

	^super queryCommand: aCommand
!

result
	^result!

result: anObject
	result := anObject!

viewName
	^viewName!

viewName: anObject
	viewName := anObject! !
!SurrogateDialog categoriesFor: #additionalAccelerators!accessing!private! !
!SurrogateDialog categoriesFor: #cancel!commands!public! !
!SurrogateDialog categoriesFor: #cancelBlock!accessing!private! !
!SurrogateDialog categoriesFor: #cancelBlock:!accessing!private! !
!SurrogateDialog categoriesFor: #containedPresenter!accessing!private! !
!SurrogateDialog categoriesFor: #containedPresenter:!accessing!private! !
!SurrogateDialog categoriesFor: #createComponents!initializing!public! !
!SurrogateDialog categoriesFor: #createSchematicWiring!initializing!public! !
!SurrogateDialog categoriesFor: #initialize!initializing!public! !
!SurrogateDialog categoriesFor: #isClosing!accessing!private! !
!SurrogateDialog categoriesFor: #isClosing:!accessing!private! !
!SurrogateDialog categoriesFor: #ok!commands!public! !
!SurrogateDialog categoriesFor: #okBlock!accessing!private! !
!SurrogateDialog categoriesFor: #okBlock:!accessing!private! !
!SurrogateDialog categoriesFor: #onCloseRequested:!event handling!public! !
!SurrogateDialog categoriesFor: #onViewOpened!event handling!public! !
!SurrogateDialog categoriesFor: #queryCommand:!commands!public! !
!SurrogateDialog categoriesFor: #result!accessing!private! !
!SurrogateDialog categoriesFor: #result:!accessing!private! !
!SurrogateDialog categoriesFor: #viewName!accessing!private! !
!SurrogateDialog categoriesFor: #viewName:!accessing!private! !

"Binary Globals"!

"Resources"!

(ResourceIdentifier class: SurrogateDialog name: 'Default view') assign: (Object fromBinaryStoreBytes:
(ByteArray fromBase64String: 'IVNUQiAxIEYCDAABAAAAVmlld1Jlc291cmNlAAAAAA4BJABTVEJSZXNvdXJjZVNUQkJ5dGVBcnJh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'))!

